# encoding: utf-8
require "logstash/filters/base"
require "logstash/namespace"
require 'uri'
require 'cgi'

Dir[File.dirname(__FILE__) + '/../helpers/*.rb'].each { |file| require file }
Dir[File.dirname(__FILE__) + '/./ezproxy_providers/*.rb'].each { |file| require file }

# This  filter will replace the contents of the default 
# message field with whatever you specify in the configuration.
#
# It is only intended to be used as an .
class LogStash::Filters::Ezproxy < LogStash::Filters::Base

  # Setting the config_name here is required. This is how you
  # configure this filter from your Logstash config.
  #
  # filter {
  #    {
  #     url => "Field containing url to parse"
  #     target => "Field to output metadata to - default is request_metadata"
  #     mime_tag => true|false set inital mime types for all requests based on file extensions - default false
  #     doc_id_tag => true|false set unit and title id from docID like params - default false
  #     path_tag => true|false add request path to metadata - default false
  #   }
  # }
  #
  config_name "ezproxy"
  
  # The url to be parsed by the filter
  config :url, :validate => :string, :required => true
  config :target, :validate => :string, :default => "request_metadata"
  config :mime_tag, :validate => :boolean, :default => false
  config :doc_id_tag, :validate => :boolean, :default => false
  config :path_tag, :validate => :boolean, :default => false

  @@hosts = {
    "www.cambridge.org" =>            lambda { |path, params, uri| EzproxyProviders::Cambridge.parse(path, params, uri) },
    "www.dawsonera.com" =>            lambda { |path, params, uri| EzproxyProviders::DawsonEra::parse(path, params) },
    "ebookcentral.proquest.com" =>    lambda { |path, params, uri| EzproxyProviders::EBookCentral::parse(path, params) },
    "ebscohost.com" =>                lambda { |path, params, uri| EzproxyProviders::Ebscohost::parse(path, params, uri) },
    "elsevierelibrary.co.uk" =>       lambda { |path, params, uri| EzproxyProviders::Elsevier::parse(path, params) },
    "emeraldinsight.com" =>           lambda { |path, params, uri| EzproxyProviders::Emerald::parse(path, params) },
    "www.ft.com" =>                   lambda { |path, params, uri| EzproxyProviders::FT::parse(path, params) },
    "galegroup.com" =>                lambda { |path, params, uri| EzproxyProviders::Gale::parse(path, params) },
    "heinonline.org" =>               lambda { |path, params, uri| EzproxyProviders::HeinOnline::parse(path, params) },
    "ieee.org" =>                     lambda { |path, params, uri| EzproxyProviders::IEEE::parse(path, params) },
    "www.jstor.org" =>                lambda { |path, params, uri| EzproxyProviders::Jstor::parse(path, params) },
    "www.lexisnexis.com" =>           lambda { |path, params, uri| EzproxyProviders::LexisNexis::parse(path, params) },
    "webanalytics.lexisnexis.com" =>  lambda { |path, params, uri| EzproxyProviders::LexisWebAnalytics::parse(path, params, uri) },
    "lib.myilibrary.com" =>           lambda { |path, params, uri| EzproxyProviders::MyILibrary::parse(path, params) },
    "www.oxfordscholarship.com" =>    lambda { |path, params, uri| EzproxyProviders::Oxford::parse(path, params) },
    "search.proquest.com" =>          lambda { |path, params, uri| EzproxyProviders::ProquestSearch::parse(path, params) },
    "journals.sagepub.com" =>         lambda { |path, params, uri| EzproxyProviders::Sage::parse(path, params) },
    "els-cdn.com" =>                  lambda { |path, params, uri|
      return EzproxyProviders::ScienceDirect::parse(path, params).merge!({ 'provider' => 'elsevier-cdn' })
    },
    "sciencedirect.com" =>            lambda { |path, params, uri|
      return EzproxyProviders::ScienceDirect::parse(path, params).merge!({ 'provider' => 'sciencedirect' })
    },
    "scopus.com" =>                   lambda { |path, params, uri| EzproxyProviders::Scopus::parse(path, params) },
    "springer.com" =>                 lambda { |path, params, uri| EzproxyProviders::Springer::parse(path, params) },
    "www.tandfonline.com" =>          lambda { |path, params, uri| EzproxyProviders::TandF::parse(path, params) },
    "thomsonreuters.com" =>           lambda { |path, params, uri|
      return EzproxyProviders::WebOfKnowledge::parse(path, params).merge!({ 'provider' => 'thomsonreuters' })
    },
    "webofknowledge.com" =>           lambda { |path, params, uri|
      return EzproxyProviders::WebOfKnowledge::parse(path, params).merge!({ 'provider' => 'webofknowledge' })
    },
    "westlaw.co.uk" =>                lambda { |path, params, uri| EzproxyProviders::Westlaw::parse(path, params) },
    "wiley.com" =>                    lambda { |path, params, uri| EzproxyProviders::Wiley::parse(path, params) }
  }
  

  public
  def register
    # Add instance variables 
  end # def register

  public
  def filter(event)
    begin
      input = URI::extract(event.get(@url))[0]
    rescue => e
      puts e.message
      puts "at"
      puts e.backtrace.inspect
      puts "for"
      puts @url
      event.tag("ezproxy_parse_failure")
    end

    data = {}

    parsed_url = URLParser::parse(input)
    uri = parsed_url['uri']
    path = parsed_url['path']
    params = parsed_url['params']

    unless uri == nil

      begin
        if @mime_tag
          data.merge!(Mime::parse(path))
        end
        if @doc_id_tag
          data.merge!(ParamHelper::parse(params))
        end

        @@hosts.each do |key, value|
          if uri.host.include?(key)
            data.merge!(value.call(path, params, uri))
          end
        end

        event.tag("ezproxy_parse_success")
      rescue => e
        puts e.message
        puts "at"
        puts e.backtrace.inspect
        puts "for"
        puts uri
        event.tag("ezproxy_parse_failure")
      end
      if @path_tag
        data['path'] = path
      end
      event.set(@target, data)
    else
      event.tag("ezproxy_parse_failure")
    end


    # filter_matched should go in the last line of our successful code
    filter_matched(event)
  end # def filter
end # class LogStash::Filters::Ezproxy
